let currentNav;

class NavigationEvents
{
    constructor() { }
    handleKeyPress(event)
    {
        //console.log('Keypress: ' + event.key);
        if (location.href.includes('/signin') && (event.key === 'Backspace' || event.key === 'Enter'))
            return;

        try
        {
            switch (event.key)
            {
                case 'BrowserBack':
                case 'Backspace':
                case 'Escape':
                    event.preventDefault();
                    navigateBack(event.key);
                    break;
                case 'ArrowRight':
                    currentNav.focusRight();
                    break;
                case 'ArrowLeft':
                    if (event.altKey)
                    {
                        event.preventDefault();
                        navigateBack(event.key);
                    }
                    else
                        currentNav.focusLeft();
                    break;
                case 'ArrowUp':
                    event.preventDefault();
                    currentNav.focusUp();
                    break;
                case 'ArrowDown':
                    event.preventDefault();
                    currentNav.focusDown();
                    break;
                case 'Enter':
                    event.preventDefault();
                    currentNav.enterPressed();
                    break;
                case '0':
                    if (!getModifierKeyPressed(event))
                    {
                        if (currentNav instanceof ProfilesNavigator)
                            currentNav.handleBack();
                        else {
                            const nav = new ProfilesNavigator(currentNav);
                            if (nav.focusFirst())
                                currentNav = nav;
                        }
                    }
                    break;
                case '1':
                    if (!getModifierKeyPressed(event))
                        location.href = "/gp/video/mystuff/";
                    break;
                case '2':
                    if (!getModifierKeyPressed(event))
                        location.href = "/gp/video/storefront/ref=atv_cat_leftpanel_movie?contentType=movie&contentId=home";
                    break;
                case '3':
                    if (!getModifierKeyPressed(event))
                        location.href = "/gp/video/storefront/ref=atv_cat_dimension_contenttype_tv_singular?contentType=tv&contentId=home";
                    break;
                default:
                    if (currentNav && currentNav.handleKeyPress)
                      currentNav.handleKeyPress(event);
                    break;
            }
        }
        catch (error) {
        }
    }
}

class ProfilesNavigator
{
    constructor(parentNav)
    {
        this.parentNavigator = parentNav;
        this.nProfile = -1;
    }
    focusFirst()
    {
        const profileButton = document.getElementById('nav-profiles-dropdown');
        if (profileButton)
            profileButton.click();
        this.focusProfile(0);
        return this.nProfile == 0;
    }
    focusRight()
    {
        this.handleBack();
    }
    focusLeft()
    {
        this.handleBack();
    }
    focusUp()
    {
        this.focusProfile(this.nProfile - 1);
    }
    focusDown()
    {
        this.focusProfile(this.nProfile + 1);
    }
    focusProfile(profileNum)
    {
        const profileList = document.querySelector('.nav-profiles-menu');
        if (profileList)
        {
            profileList.focus();
            const profiles = profileList.querySelectorAll('form');
            console.log('***Parent ' + this.parentNavigator);
            if (profileNum >= 0 && profileNum < profiles.length)
            {
                this.nProfile = profileNum;
                switchFocus(getCurrent(), profiles[this.nProfile]);
            }
        }
    }
    handleBack()
    {
        const profileButton = document.getElementById('nav-profiles-dropdown');
        if (profileButton)
        {
            profileButton.click();
            currentNav = this.parentNavigator;
            currentNav.focusFirst();
        }
    }
    enterPressed()
    {
        const current = getCurrent();
        if (current)
            current.submit();
    }
}

class BrowseGridNavigator
{
    constructor()
    {
        this.nRow = 0;
        this.nColumn = 0;
        this.nSubRow = -1;
    }
    focusFirst()
    {
        // Re-select previous from cookie
        const selection = getPreviousSelection();
        if (selection)
        {
            console.log(selection);
            const values = selection.split('_');
            this.nRow = parseInt(values[0]);
            this.nColumn = parseInt(values[1]);
            this.nSubRow = parseInt(values[2]);
        }
        this.selectGridItem(this.nRow, this.nColumn, 1);
    }
    selectGridItem(rowNum, columnNum, direction)
    {
        const rows = document.querySelectorAll('.tst-collection');
        if (rowNum >= 0 && rowNum < rows.length)
        {
            this.nRow = rowNum;

            // Skip "special" rows
            if (!rows[rowNum].querySelector('.tst-faceted-carousel'))
            {
                if (direction == 1)
                    this.focusDown();
                else
                    this.focusUp();
                return;
            }

            const columns = rows[this.nRow].getElementsByTagName('li');
            if (columnNum >= 0 && columnNum < columns.length)
            {
                this.nColumn = columnNum;
                //console.log('***Selecting grid ' + this.nRow + ', ' + this.nColumn);
                const cards = columns[this.nColumn].querySelectorAll('.tst-title-card');
                let selection;
                if (cards.length < 2)
                {
                    selection = cards[0];
                    this.nSubRow = -1;
                }
                else // double-decker row
                {
                    if (this.nSubRow < 0)
                        this.nSubRow = direction == 1 ? 0 : 1;
                    selection = cards[this.nSubRow];
                }
                switchFocus(getCurrent(), selection);
            }
        }
    }
    focusRight()
    {
        this.selectGridItem(this.nRow, this.nColumn + 1, 0);
    }
    focusLeft()
    {
        if (this.nColumn == 0)
            navigateBack('');
        else
            this.selectGridItem(this.nRow, this.nColumn - 1, 0);
    } 
    focusUp()
    {
        if (this.nRow == 0)
            focusMainMenu(this);
        else if (this.nSubRow != 1)
            this.selectGridItem(this.nRow - 1, 0, -1);
        else
        {
            this.nSubRow = 0;
            this.selectGridItem(this.nRow, this.nColumn, -1);
        }
    }
    focusDown()
    {
        if (this.nSubRow != 0)
            this.selectGridItem(this.nRow + 1, 0, 1);
        else
        {
            this.nSubRow = 1;
            this.selectGridItem(this.nRow, this.nColumn, 1);
        }
    }
    enterPressed()
    {
        const current = getCurrent();
        if (current)
        {
            const link = current.querySelector('a');
            if (link)
            {
                setPreviousSelection(this.nRow + '_' + this.nColumn + '_' + this.nSubRow);
                window.location.href = link.href;
            }
        }

    }
}

class DetailsNavigator
{
    constructor()
    {
        this.aryButtons = new Array();
        const playButton = document.querySelector('.dv-dp-node-playback');
        if (playButton)
            this.aryButtons.push(playButton);
        const trailerButton = document.querySelector('.dv-dp-node-trailer');
        if (trailerButton)
            this.aryButtons.push(trailerButton);
        const listButton = document.querySelector('.dv-dp-node-watchlist');
        if (listButton)
            this.aryButtons.push(listButton);
        this.nButton = 0;
    }
    focusFirst()
    {
        this.focusButton(this.nButton);
    }
    focusRight()
    {
        this.focusButton(this.nButton + 1);
    }
    focusLeft()
    {
        if (this.nButton == 0)
            navigateBack('');
        else
            this.focusButton(this.nButton - 1);
    }
    focusUp()
    {
        this.focusSeasons();
    }
    focusDown()
    {
        const episodesNav = new EpisodesNavigator(this);
        if (episodesNav.focusFirst())
            currentNav = episodesNav;
    }
    focusButton(buttonNum)
    {
        if (buttonNum >= 0 && buttonNum < this.aryButtons.length)
        {
            this.nButton = buttonNum;
            switchFocus(getCurrent(), this.aryButtons[this.nButton]);
        }
    }
    focusSeasons()
    {
        const seasonsButton = document.getElementById('av-droplist-av-atf-season-selector');
        if (seasonsButton)
        {
            seasonsButton.click();
            currentNav = new SeasonsNavigator(this);
            currentNav.focusFirst();
        }
        else
            focusMainMenu(this);
    }
    enterPressed()
    {
        const current = getCurrent();
        if (current)
        {
            const link = current.querySelector('a');
            if (link)
            {
                link.click();
                currentNav = new PlayerNavigator(this);
            }
            else
            {
                const button = current.querySelector('button');
                if (button)
                    button.click();
            }
        }

    }
}

class EpisodesNavigator
{
    constructor(parentNav)
    {
        this.parentNavigator = parentNav;
        this.nEpisode = -1;
    }
    focusFirst()
    {
        this.focusEpisode(0);
        return this.nEpisode == 0;
    }
    focusRight() { }
    focusLeft()
    {
        navigateBack('');
    }
    focusUp()
    {
        if (this.nEpisode > 0)
            this.focusEpisode(this.nEpisode - 1);
        else
        {
            currentNav = this.parentNavigator;
            currentNav.focusFirst();
        }
    }
    focusDown()
    {
        this.focusEpisode(this.nEpisode + 1);
    }
    focusEpisode(episodeNum)
    {
        const episodes = document.querySelectorAll('.js-node-episode-container');
        if (episodeNum >= 0 && episodeNum < episodes.length)
        {
            this.nEpisode = episodeNum;
            const episode = episodes[this.nEpisode].querySelector('.js-eplist-episode');
            switchFocus(getCurrent(), episode);
        }

    }
    enterPressed()
    {
        const current = getCurrent();
        if (current)
        {
            const link = current.querySelector('a');
            if (link)
            {
                link.click();
                currentNav = new PlayerNavigator(this);
            }
        }
    }
}

class SeasonsNavigator
{
    constructor(parentNav)
    {
        this.parentNavigator = parentNav;
        this.nSeason = -1;
    }
    focusFirst()
    {
        this.focusSeason(0);
    }
    focusRight()
    {
        this.handleBack();
    }
    focusLeft()
    {
        this.handleBack();
    }
    focusUp()
    {
        if (this.nSeason < 1)
        {
            this.handleBack();
            focusMainMenu(this.parentNavigator);
        }
        else
            this.focusSeason(this.nSeason - 1);
    }
    focusDown()
    {
        this.focusSeason(this.nSeason + 1);
    }
    focusSeason(seasonNum)
    {
        const seasonsDiv = document.querySelector('.dv-node-dp-seasons');
        if (seasonsDiv)
        {
            seasonsDiv.focus();
            const seasons = seasonsDiv.querySelectorAll('li');
            if (seasonNum >= 0 && seasonNum < seasons.length)
            {
                this.nSeason = seasonNum;
                switchFocus(getCurrent(), seasons[this.nSeason]);
            }
        }
    }
    handleBack()
    { 
        const seasonsButton = document.getElementById('av-droplist-av-atf-season-selector');
        if (seasonsButton)
        {
            seasonsButton.click();
            currentNav = this.parentNavigator;
            currentNav.focusFirst();
        }
    }
    enterPressed()
    {
        const current = getCurrent();
        if (current)
        {
            const link = current.querySelector('a');
            if (link)
                link.click();
        }
    }
}

class MainMenuNavigator
{
    constructor(parentNav)
    {
        this.parentNavigator = parentNav;
        this.nItem = 0;
    }
    focusFirst()
    {
        this.focusItem(this.nItem);
    }
    focusRight() 
    {
        this.focusItem(this.nItem + 1);
    }
    focusLeft() 
    {
        this.focusItem(this.nItem - 1);
    }
    focusUp() 
    {
    }
    focusDown() 
    {
        currentNav = this.nItem != 4 ? this.parentNavigator : new CategoriesNavigator(this);
        currentNav.focusFirst();
    }
    focusItem(itemNum) 
    {
        const menuDiv = document.querySelector('.av-retail-m-nav-lists');
        if (menuDiv) 
        {
            menuDiv.focus();
            const items = menuDiv.querySelectorAll('.av-retail-m-nav-main-item');
            if (itemNum >= 0 && itemNum < items.length) 
            {
                this.nItem = itemNum;
                switchFocus(getCurrent(), items[this.nItem].firstElementChild);
            }
        }
    }
    enterPressed() 
    {
        const link = getCurrent();
        if (link) 
            link.click();
    }
}

class CategoriesNavigator
{
    constructor(parentNav)
    {
        this.parentNavigator = parentNav;
        this.nItem = 0;
        this.nColumn = 0;
        this.aryRowCount = [3, 9, 5];
    }
    focusFirst()
    {
        this.focusItem(this.nColumn, this.nItem);
    }
    focusRight() 
    {
        if (this.nColumn == 0)
        {
            if ((this.nItem + 1) % this.aryRowCount[0] != 0)
                this.focusItem(this.nColumn, this.nItem + 1);
            else
                this.focusItem(this.nColumn + 1, 0);
        }
        else
        {
            if (this.nItem >= this.aryRowCount[this.nColumn])
                this.focusItem(this.nColumn + 1, 0);
            else
                this.focusItem(this.nColumn, this.nItem + this.aryRowCount[this.nColumn]);
        }
    }
    focusLeft() 
    {
        if (this.nColumn == 0 && this.nItem % this.aryRowCount[0] != 0)
            this.focusItem(this.nColumn, this.nItem - 1);
        else
        {
            if (this.nItem < this.aryRowCount[this.nColumn])
                this.focusItem(this.nColumn - 1, 0);
            else
                this.focusItem(this.nColumn, this.nItem - this.aryRowCount[this.nColumn]);
        }
    }
    focusUp() 
    {
        if (this.nColumn == 0)
        {
            if (this.nItem < this.aryRowCount[0])
                this.focusParent();
            else
                this.focusItem(0, this.nItem - this.aryRowCount[0]);
        }
        else
        {
            if (this.nItem % this.aryRowCount[this.nColumn] == 0)
                this.focusParent();
            else
                this.focusItem(this.nColumn, this.nItem - 1);
        }
    }
    focusDown() 
    {
        if (this.nColumn == 0)
            this.focusItem(0, this.nItem + this.aryRowCount[0]);
        else
            this.focusItem(this.nColumn, this.nItem + 1);
    }
    focusParent() 
    {
        currentNav = this.parentNavigator;
        currentNav.focusFirst();
    }
    focusItem(columnNum, itemNum) 
    {
        if (columnNum > this.ColCountArry - 1)
            return;

        var list;
        if (columnNum == 0)
            list = document.querySelector('.cat_panels');
        else   
            list = document.querySelector('.cat_columns--' + (columnNum + 1));

        if (list)
        {
            this.nColumn = columnNum;
            const items = list.getElementsByTagName('li');
            if (itemNum >= 0 && itemNum < items.length)
            {
                this.nItem = itemNum;
                switchFocus(getCurrent(), items[itemNum]);
            }
        }
    }
    enterPressed() 
    {
        const links = getCurrent().getElementsByTagName('a');
        if (links.length > 0)
            links[0].click();
    }
}

class MyStuffNavigator
{
    constructor()
    {
        this.nTab = 0;
        this.nItem = -1;
        const gridContainer = document.getElementById('aiv-cl-main-middle');
        if (gridContainer && gridContainer.firstElementChild)
            this.gridColCount = window.getComputedStyle(gridContainer.firstElementChild).gridTemplateColumns.split(" ").length;
        else
            this.gridColCount = 1;
    }
    focusFirst()
    {
        const selection = getPreviousSelection();
        if (selection)
            this.nItem = parseInt(selection);

        if (this.nItem >= 0)
        {
            const items = document.querySelectorAll('.av-hover-wrapper');
            if (this.nItem < items.length)
                switchFocus(getCurrent(), items[this.nItem]);
            else
                this.nItem = -1;
        }
        else
            this.focusTab(0);
    }
    focusRight()
    {
        const current = getCurrent();
        if (this.nItem < 0) // tabs at top
        {
            if (this.nTab == 1 || this.nTab == 4)
                return;

            this.focusTab(this.nTab + 1);
        }
        else // grid of videos
        {
            if ((this.nItem + 1) % this.gridColCount != 0)
                this.focusItem(this.nItem + 1);
        }
    }
    focusLeft() 
    {
        const current = getCurrent();
        if (this.nItem < 0) // tabs at top
        {
            if (this.nTab == 0 || this.nTab == 2)
            {
                navigateBack('');
                return;
            }

            this.focusTab(this.nTab - 1);
        }
        else // grid of videos
        {
            if (this.nItem % this.gridColCount != 0)
                this.focusItem(this.nItem - 1);
            else
                navigateBack('');
        }
    }
    focusUp()
    {
        const current = getCurrent();
        if (this.nItem < 0) // tabs at top
        {
            if (this.nTab < 2)
                focusMainMenu(this);
            else
                this.focusTab(0);
        }
        else // grid of videos
        {
            if (this.nItem >= this.gridColCount)
                this.focusItem(this.nItem - this.gridColCount);
            else
            {
                this.nItem = -1;
                this.focusTab(2);
            }
        }
    }
    focusDown()
    {
        const current = getCurrent();
        if (this.nItem < 0) // tabs at top
        {
            if (this.nTab > 1)
            {
                const gridItem = document.querySelector('.av-hover-wrapper');
                this.nItem = 0;
                switchFocus(current, gridItem);
            }
            else
                this.focusTab(2);
        }
        else // grid of videos
            this.focusItem(this.nItem + this.gridColCount);
    }
    focusTab(tabNum)
    {
        const tabs = document.querySelectorAll('.dvui-tab');
        if (tabNum >= 0 && tabNum < tabs.length)
        {
            this.nTab = tabNum;
            switchFocus(getCurrent(), tabs[this.nTab]);
        }
    }
    focusItem(itemNum)
    {
        const items = document.querySelectorAll('.av-hover-wrapper');
        if (itemNum >= 0 && itemNum < items.length)
        {
            this.nItem = itemNum;
            switchFocus(getCurrent(), items[this.nItem]);
        }

    }
    enterPressed()
    {
        var current = getCurrent();
        if (current)
        {
            const links = current.getElementsByTagName('a');
            if (links.length > 0)
            {
                current = links[0];
                setPreviousSelection(this.nItem);
            }  
            current.click()
        }
    }
}

class PlayerNavigator
{
    constructor(parentNav)
    {
        this.parentNavigator = parentNav;

    }
    focusFirst()
    {
        const player = document.querySelector('.atvwebplayersdk-player-container');
        if (player)
            player.focus();
    }
    focusRight() { }
    focusLeft() { }
    focusUp() { }
    focusDown() { }

    dispatchKey(keyCode, isKeyDown = true)
    {
        let event = new KeyboardEvent(isKeyDown ? 'keydown' : 'keyup', {
            key: String.fromCharCode(keyCode),
            keyCode: keyCode, bubbles: true, cancelable: true, view: window
        });
        const player = document.querySelector('.atvwebplayersdk-player-container');
        if (player)
        {
            //console.log('***Dispatching keydown');
            player.dispatchEvent(event);
        }
    }

    handleKeyPress(event)
    {
        //console.log('****KEY: ' + event.key);
        switch (event.key)
        {
            case 'MediaPlayPause':
                this.playPause();
                break;
            case 'MediaStop':
                this.closePlayer();
                break;
        }
    }

    enterPressed()
    {
        this.playPause();
    }

    handleBack()
    {
        this.closePlayer();
        currentNav = this.parentNavigator;
    }

    closePlayer()
    {
        const button = document.querySelector('.atvwebplayersdk-playerclose-button');
        if (button)
            button.click();
    }

    playPause()
    {
        const button = document.querySelector('.atvwebplayersdk-playpause-button');
        if (button)
        {
            button.focus();
            button.click();
        }
    }

    handleMessage(event)
    {
        //console.log('***Message: ' + event.data);
        switch (event.data)
        {
            case 'MCC_PLAY':
                this.playPause();
                break;
            case 'MCC_STOP':
                this.handleBack();
                break;
            case 'MCC_FAST_FORWARD':
                this.dispatchKey(39);
                break;
            case 'MCC_REWIND':
                this.dispatchKey(37);
                break;
        }
    }
}

function getPreviousSelection()
{
    const cookie = document.cookie.split('; ').find((row) => row.startsWith('mcselect='));
    if (cookie)
        return cookie.split('=')[1];
    else
        return null;
}

function setPreviousSelection(selection)
{
    document.cookie = 'mcselect=' + selection + ';max-age=86400';
}

function clearPreviousSelection()
{
    document.cookie = "mcselect=; expires=Thu, 01 Jan 1970 00:00:00 UTC"
}

function getCurrent()
{
    return document.querySelector('.mctv-selected');
}

function focusMainMenu(parent)
{
    clearPreviousSelection();
    currentNav = new MainMenuNavigator(parent);
    currentNav.focusFirst();
}

function switchFocus(current, next)
{
    if (next)
    {
        if (current)
        {
            current.classList.remove('mctv-selected');
            current.tabIndex = -1;
            const tt = current.querySelector('.dvui-tooltip');
            if (tt)
                tt.style.visibility = 'hidden';
        }
        next.classList.add('mctv-selected');
        next.tabIndex = 0;
        next.focus();
        const ttNext = next.querySelector('.dvui-tooltip');
        if (ttNext && location.href.includes('/detail/'))
            ttNext.style.visibility = 'visible';
    }
}

function pageChanged()
{
    setCurrentNavigator();
}

function setCurrentNavigator()
{
    if (location.href.includes('/detail/'))
        currentNav = new DetailsNavigator();
    else if (location.href.includes('/mystuff/'))
        currentNav = new MyStuffNavigator();
    else
        currentNav = new BrowseGridNavigator();

    currentNav.focusFirst();
}

function navigateBack(key)
{
    if (currentNav && currentNav.handleBack)
        currentNav.handleBack();
    else
    {
        clearPreviousSelection();
        if (!window.location.href.includes('node='))
        {
            if (key != 'BrowserBack') // Cannot prevent BrowserBack, so don't repeat
                window.history.back();
        }
        else
        {
            let jsonObject = { Key: 'jrweb_back' };
            window.chrome.webview.postMessage(jsonObject);
        }
    }
}

function getModifierKeyPressed(event)
{
    return event.altKey || event.ctrlKey || event.metaKey || event.shiftKey;
}

var styles = `
html {
    overflow: scroll;
    overflow-x: hidden;
}
::-webkit-scrollbar {
    width: 0;
    background: transparent;
}
.mctv-selected {
    border: 4px solid blue;
    outline: 0px;
}
`

const navEvents = new NavigationEvents;
window.addEventListener('load', async () => {
    var styleSheet = document.createElement("style");
    styleSheet.type = "text/css";
    styleSheet.innerText = styles;
    document.head.appendChild(styleSheet);

    window.addEventListener("keydown", function (e) {
        if ([37, 38, 39, 40].indexOf(e.keyCode) > -1) {
            e.preventDefault();
        }
        navEvents.handleKeyPress(e);
    }, false);
    pageChanged();
}, { once: true });

// catch URL changes
window.addEventListener('popstate', () => {
    window.dispatchEvent(new Event('locationchange'));
});

history.pushState = ( f => function pushState(){
    var ret = f.apply(this, arguments);
    window.dispatchEvent(new Event('locationchange'));
    return ret;
})(history.pushState);

history.replaceState = ( f => function replaceState(){
    var ret = f.apply(this, arguments);
    window.dispatchEvent(new Event('locationchange'));
    return ret;
})(history.replaceState);

window.addEventListener('locationchange', async () => {
    pageChanged();
});
try {
    window.chrome.webview.addEventListener('message', function (e) {
        if (currentNav && currentNav.handleMessage)
            currentNav.handleMessage(e);
    });
} catch (error) {}
